/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkBlackBoxInputConnector.cxx,v $
  Language:  C++
  Date:      $Date: 2008/12/11 09:50:35 $
  Version:   $Revision: 1.9 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 *  \file 
 *  \brief Class bbtk::BlackBoxInputConnector : 
 */
#include "bbtkBlackBoxInputConnector.h"
#include "bbtkMessageManager.h"
#include "bbtkBlackBox.h"

namespace bbtk
{
  //========================================================================
  BlackBoxInputConnector::BlackBoxInputConnector(BlackBox::Pointer b)  
    : mBox(b), mConnection(0), mStatus(MODIFIED)
  {
    bbtkDebugMessage("Kernel",9,"BlackBoxInputConnector::BlackBoxInputConnector()"<<std::endl);
  }
  //========================================================================

  //========================================================================
  BlackBoxInputConnector::~BlackBoxInputConnector() 
  {
    bbtkDebugMessageInc("Kernel",9,"BlackBoxInputConnector::~BlackBoxInputConnector()"<<std::endl);
    bbtkDebugDecTab("Kernel",9);
  }
  //========================================================================

  //========================================================================
  void BlackBoxInputConnector::SetConnection(Connection* c) 
  { 
    bbtkDebugMessage("Kernel",9,"BlackBoxInputConnector::SetConnection("<<c<<")"<<std::endl);
    mConnection = c; 
  }
  //========================================================================

  //========================================================================
  void BlackBoxInputConnector::UnsetConnection(Connection* c) 
  { 
    bbtkDebugMessage("Kernel",9,"BlackBoxInputConnector::UnsetConnection("
		     <<c<<")"<<std::endl);
    mConnection = 0; 
  }
  //========================================================================
  
  //========================================================================
  void BlackBoxInputConnector::RecursiveExecute()
  {
    // If connected and OUTOFDATE : recursive update
    // Post-update status is updated by the connection 
    // (either MODIFIED or OUTOFDATE)
    if ( mConnection && (mStatus == OUTOFDATE) )
      {
	mConnection->RecursiveExecute();
      }
    else
      {
	bbtkDebugMessage("process",5,
			 "--> BlackBoxInputConnector::RecursiveExecute() : "
			 <<"No connection or input not Out-of-date : nothing to do"
			 <<std::endl);

      }
  }
  //========================================================================

}
// namespace bbtk

