/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkItkBlackBoxMacros.h,v $
  Language:  C++
  Date:      $Date: 2008/10/17 08:18:13 $
  Version:   $Revision: 1.4 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         



/**
 *  \file 
 *  \brief Defines macros for the creation of itk object inherited black boxes
 */
#ifndef __bbtkItkBlackBoxMacros_h__
#define __bbtkItkBlackBoxMacros_h__
//===========================================================================
//============================================================================
// ITK Specific macros
//===========================================================================
//===========================================================================

//===========================================================================
#define BBTK_ITK_DELETE()						\
  void bbDelete() {							\
    itkParent::UnRegister();						\
  }
//===========================================================================

//===========================================================================
#define BBTK_ITK_BLACK_BOX_INTERFACE(CLASS,PARENTBLACKBOX,ITKPARENT)	\
  BBTK_BLACK_BOX_INTERFACE(CLASS,PARENTBLACKBOX);			\
  typedef ITKPARENT itkParent;						\
  BBTK_ITK_DELETE();
//===========================================================================

//===========================================================================
/// Declares an itk-inherited AtomicBlackBox input 
#define BBTK_DECLARE_ITK_INPUT(NAME,TYPE)				\
  public:								\
  TYPE bbGetInput##NAME ()						\
  { return itkParent::GetInput(); }					\
  void bbSetInput##NAME (TYPE d)					\
  { itkParent::SetInput(d);						\
    /*bbSetModifiedStatus();*/ }						       
//===========================================================================

//===========================================================================
#define BBTK_DECLARE_ITK_OUTPUT(NAME,TYPE)			\
  public:								\
  TYPE bbGetOutput##NAME ()						\
  { return itkParent::GetOutput(); }					\
  void bbSetOutput##NAME (TYPE d)					\
  { /*PARENT::GetOutput() = d;*/ }					
//===========================================================================

//===========================================================================
/// Declares an AtomicBlackBox input corresponding to an inherited itk parameter
/// which was declared by itkSetMacro/itkGetMacro
/// The NAME **MUST** be the same than the itk parameter name
#define BBTK_DECLARE_ITK_PARAM(NAME,TYPE)			\
  public:								\
  TYPE bbGetInput##NAME ()						\
  { return itkParent::Get##NAME(); }					\
  void bbSetInput##NAME (TYPE d)					\
  { itkParent::Set##NAME(d);						\
    /*bbSetModifiedStatus();*/ }
//===========================================================================

//============================================================================
/// Defines the default bbUserProcess method for itk inherited black boxes
/// (calls itkParent::Update)
#define BBTK_ITK_PROCESS()						\
  public:								\
  inline void bbUserProcess()						\
  {									\
    bbtkDebugMessage("process",1,"**> Processing ["<<bbGetFullName()	\
		     <<"]"<<std::endl);					\
    itkParent::Update();						\
    bbtkDebugMessage("process",2,"<** Processing ["<<bbGetFullName()	\
		     <<"]"<<std::endl);					\
  }
//============================================================================

//===========================================================================
/// EOF
//===========================================================================
#endif
