/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkKWBlackBox.cxx,v $
  Language:  C++
  Date:      $Date: 2008/12/18 11:54:44 $
  Version:   $Revision: 1.8 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         
/**
 *  \file 
 *  \brief 
 */

#ifdef USE_KWWIDGETS




 
#include "bbtkKWBlackBox.h"
#include "bbtkBlackBoxOutputConnector.h"
#include "vtkKWBlackBoxDialog.h"


namespace bbtk
{

  //=========================================================================
  // KWBlackBox
  //=========================================================================

  //=========================================================================
  //=========================================================================
  //=========================================================================
  //=========================================================================
  BBTK_BLACK_BOX_IMPLEMENTATION(KWBlackBox,AtomicBlackBox);
  //=========================================================================
  
  //=========================================================================
  void KWBlackBox::bbUserConstructor()
  {
    bbtkDebugMessage("Kernel",9,"KWBlackBox::bbUserConstructor()"<<std::endl);
    bbInitAttributes();
  }
  //=========================================================================

  //=========================================================================
  void KWBlackBox::bbUserCopyConstructor(bbtk::BlackBox::Pointer)
  {
    bbtkDebugMessage("Kernel",9,"KWBlackBox::bbUserCopyConstructor()"
		     <<std::endl);
    bbInitAttributes();
  }
  //=========================================================================


  //=========================================================================
  void KWBlackBox::bbUserDestructor()
  {
    bbtkDebugMessage("kw",9,"==> KWBlackBox::bbUserDestructor() ["<<bbGetFullName()<<"]"<<std::endl);
    if (bbGetWindow()) {
      bbGetWindow()->bbClose();
      bbSetWindow(0);
    }
    bbtkDebugMessage("kw",9,"<== KWBlackBox::bbUserDestructor() ["<<bbGetFullName()<<"]"<<std::endl);
  }
  //=========================================================================
  


  //=========================================================================
  /**
   * \brief Initialize the attributes of the class
   *
   */
  void KWBlackBox::bbInitAttributes()
  {
    bbmWindow = 0;
    //    bbmWidget = 0;
    //    bbSetInputWinParent(0);
    bbSetInputWinTitle(bbGetName());
    bbSetInputWinWidth(800);
    bbSetInputWinHeight(800);
    bbSetInputWinDialog(false);
    bbSetOutputWidget(0);

    //    bbSetWidgetEventHandler(0);
    //  bbSetUpdateTransferedToParent(false);
  }
  //=========================================================================


  //=========================================================================
  void KWBlackBox::bbProcess()
  { 

    this->bbUserProcess(); 

    // If output widget not connected create the window 
    if ( (*bbGetOutputConnectorMap().find("Widget")).second
	 ->GetConnectionVector().size() == 0 ) 
      {
	Window* show = 0;
	// If the window already exists : no need creating it
	if (bbGetWindow()!=0)
	  {
	    bbtkDebugMessage("kw",2,
			     "-> Window already exists"
			     <<std::endl);
	    show = bbGetWindow();
	  }
	// Else create window 
	else 
	  {
	    bbtkDebugMessage("kw",2,
			     "-> Creating the window"
			     <<std::endl);
	    KW::GetApplication();
	    
	    // Input WinDialog set to true : creating a Dialog
	    /*
	    if (bbGetInputWinDialog()) 
	      {
		bbtkDebugMessage("kw",2,
				 "   Input WinDialog set to true : creating a Dialog"
				 <<std::endl);
	    */
		vtkKWBlackBoxDialog* win = vtkKWBlackBoxDialog::New();
		show = (Window*) win;
		win->bbSetBlackBox( GetThisPointer<KWBlackBox>());
		KW::GetApplication()->AddWindow((vtkKWWindowBase*)win);
		win->Create();
		win->SetResizable(0,0);

		bbUserCreateWidget(win->GetFrame());
		/*
		KW::GetApplication()->Script
		  ("pack %s -expand yes -fill both",
		   bbGetOutputWidget()->GetWidgetName());
		*/
		
		KW::GetApplication()->Script("place %s -x 0 -y 0 -width %d -height %d",
					     bbGetOutputWidget()->GetWidgetName(),
					     bbGetInputWinWidth(),
					     bbGetInputWinHeight());
		
		



		/*
		bbGetOutputWidget()->SetParent(win);
		bbGetOutputWidget()->Create();
		KW::GetApplication()->Script("pack %s -side left -anchor c -expand y",
			
				     //"pack %s -side top -anchor nw -expand y -fill none -padx 2 -pady 2", 
					     bbGetOutputWidget()->GetWidgetName());
		*/			     
		/*	
		  }
	
	    // Input WinDialog set to false : creating a Frame
	    else 
	      {
		bbtkDebugMessage("process",2,
				 "   Input WinDialog set to false : creating a Frame"
				 <<std::endl);
		vtkKWBlackBoxFrame* win = vtkKWBlackBoxFrame::New();
		show = (Window*) win;
		win->bbSetBlackBox( GetThisPointer<KWBlackBox>());
		//		win->SetName( bbGetInputWinTitle() + " - bbtk (c) CREATIS LRMN" );
		//		win->SetWidth( bbGetInputWinWidth() );
		//		win->SetHeight( bbGetInputWinHeight() );
		KW::GetApplication()->AddWindow((vtkKWWindowBase*)win);
		win->Create();
		
		bbGetOutputWidget()->SetParent(win);
		bbGetOutputWidget()->Create();
		KW::GetApplication()->Script("pack %s -side top -anchor nw -expand y -fill none -padx 2 -pady 2", 
					     bbGetOutputWidget()->GetWidgetName());	      }
	*/
	
		//	win->Invoke();
	  }
	
	// Show the window

	show->bbShow(); 
	//	std::cout << "KW::GetApplication()->Start();"<<std::endl;
	//	KW::GetApplication()->Start();
	//	std::cout << "AFTER KW::GetApplication()->Start();"<<std::endl;
	
      }		  
    //   
    
  }
  //=========================================================================
  

  //==================================================================
  vtkKWWidget*  KWBlackBox::bbCreateWidgetOfInput(const std::string& in, vtkKWFrame* parent)
  {
    vtkKWWidget* w = 0;
    // If input is connected 
    BlackBoxInputConnector* c = bbGetInputConnectorMap().find(in)->second ;
    if ( c->IsConnected() )			
      {
	// Get black box from 
	BlackBox::Pointer from = 
	  c->GetConnection()->GetBlackBoxFrom();
	// Cast it into a KWBlackBox
	KWBlackBox::Pointer wfrom = boost::dynamic_pointer_cast<KWBlackBox>(from);
	// Call bbCreateWidget
	wfrom->bbUserCreateWidget(parent);
	// Get the widget created
	w = wfrom->bbGetOutputWidget();
      }
    return w;
  }
  //==================================================================


  //==================================================================
   void KWBlackBox::bbHideWindow()
  {
    bbtkDebugMessageInc("kw",1,"=> KWBlackBox::bbHideWindow() ["
			<<bbGetFullName()<<"]"<<std::endl);

    if (bbGetWindow()!=0) bbGetWindow()->bbHide();

    bbtkDebugMessageDec("kw",2,"<= KWBlackBox::bbHideWindow() ["
			<<bbGetFullName()<<"]"<<std::endl);
  }
  //==================================================================


  //==================================================================
   void KWBlackBox::bbCloseWindow()
  {
    bbtkDebugMessageInc("kw",1,"=> KWBlackBox::bbCloseWindow() ["
			<<bbGetFullName()<<"]"<<std::endl);

    if (bbGetWindow()!=0) bbGetWindow()->bbClose();

    bbtkDebugMessageDec("kw",2,"<= KWBlackBox::bbCloseWindow() ["
			<<bbGetFullName()<<"]"<<std::endl);
  }
  //==================================================================

  //==================================================================
  KWBlackBox::Window* KWBlackBox::bbGetContainingWindow()
  {
    if (bbGetWindow()!=0) return bbGetWindow();
    BlackBox::OutputConnectorMapType::const_iterator i 
      = bbGetOutputConnectorMap().find("Widget");
    if ( i->second->GetConnectionVector().size() != 0 ) 
      {
	return boost::static_pointer_cast<KWBlackBox>
	  (i->second->GetConnectionVector().front() //.lock()
	   ->GetBlackBoxTo())->bbGetContainingWindow();
      }
    return 0;
  }
  //==================================================================


  //==================================================================
  bool KWBlackBox::bbIsShown()
  {
    if (bbGetContainingWindow()!=0)
      return bbGetContainingWindow()->bbIsShown();
    return false;
  }
  //==================================================================


}//namespace bbtk


#endif

