/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkVtkBlackBoxMacros.h,v $
  Language:  C++
  Date:      $Date: 2008/10/17 08:18:14 $
  Version:   $Revision: 1.7 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         



/**
 *  \file 
 *  \brief Defines macros for the creation of vtk object inherited black boxes
 */
#ifndef __bbtkVtkBlackBoxMacros_h__
#define __bbtkVtkBlackBoxMacros_h__
//#include <vtkGarbageCollector.h>
//#include <vtkObjectFactory.h>
#include <vtkDebugLeaks.h>


//===========================================================================
//============================================================================
// VTK Specific macros
//===========================================================================
//===========================================================================


//===========================================================================
#define BBTK_VTK_DELETE()						\
  void bbDelete() {							\
    bbGetOutputOut()->SetSource(NULL);					\
    vtkParent::Delete();						\
  }

//    bbmDescriptorPointer.reset();		
//===========================================================================

//===========================================================================
#define BBTK_VTK_NEW(CLASS,VTKPARENT)					\
  inline static Pointer New(const std::string& name)			\
  {									\
    vtkDebugLeaks::ConstructClass(#VTKPARENT);				\
    bbtkDebugMessage("object",1,"##> "<<#CLASS				\
		     <<"::New(\""<<name<<"\")"<<std::endl);		\
    Pointer p = MakeBlackBoxPointer(new Self(name));			\
    bbtkDebugMessage("object",1,"<## "<<#CLASS				\
		     <<"::New(\""<<name<<"\")"<<std::endl);		\
    return p;								\
  }									\
//===========================================================================

//===========================================================================
#define BBTK_VTK_CLONE(CLASS,VTKPARENT)					\
  inline bbtk::BlackBox::Pointer bbClone(const std::string& name)	\
  {									\
    vtkDebugLeaks::ConstructClass(#VTKPARENT);				\
    bbtkDebugMessage("object",1,"##> "<<#CLASS				\
		     <<"::bbClone(\""<<name<<"\")"<<std::endl);		\
    Pointer p = MakeBlackBoxPointer(new Self(*this,name));		\
    bbtkDebugMessage("object",1,"<## "<<#CLASS				\
		     <<"::bbClone(\""<<name<<"\")"<<std::endl);		\
    return p;								\
  }									\


//===========================================================================
#define BBTK_VTK_BLACK_BOX_INTERFACE(CLASS,PARENTBLACKBOX,VTKPARENT)	\
  BBTK_BLACK_BOX_INTERFACE_WITHOUT_NEW(CLASS,PARENTBLACKBOX);		\
  typedef VTKPARENT vtkParent;						\
  BBTK_VTK_DELETE();							\
  public :								\
  BBTK_VTK_NEW(CLASS,VTKPARENT);					\
  BBTK_VTK_CLONE(CLASS,VTKPARENT);							
//===========================================================================

//===========================================================================
/// Declares a vtkImageAlgorithm-inherited AtomicBlackBox input 
#define BBTK_DECLARE_VTK_IMAGE_ALGORITHM_INPUT(NAME,TYPE)		\
  public:								\
  TYPE bbGetInput##NAME ()						\
  { return GetImageDataInput(0); /*vtkParent::GetInput();*/ }		\
  void bbSetInput##NAME (TYPE d)					\
  { vtkParent::SetInput( (vtkDataObject*) d);				\
    /*bbSetModifiedStatus();*/ }				
//===========================================================================

//===========================================================================
/// Declares a vtkPolyDataAlgorithm-inherited AtomicBlackBox input 
#define BBTK_DECLARE_VTK_POLY_DATA_ALGORITHM_INPUT(NAME,TYPE)		\
  public:								\
  TYPE bbGetInput##NAME ()						\
  { return GetPolyDataInput(0); /*vtkParent::GetInput();*/ }		\
  void bbSetInput##NAME (TYPE d)					\
  { vtkParent::SetInput( (vtkDataObject*) d);				\
    /*bbSetModifiedStatus();*/ }						       
//===========================================================================

//===========================================================================
/// Declares a vtkImageAlgorithm-inherited AtomicBlackBox output 
#define BBTK_DECLARE_VTK_OUTPUT(NAME,TYPE)				\
  public:								\
  TYPE bbGetOutput##NAME ()						\
  { return vtkParent::GetOutput(); }					\
  void bbSetOutput##NAME (TYPE d)					\
  { /*vtkParent::GetOutput() = d;*/ }					
//===========================================================================

//===========================================================================
/// Declares a vtkAlgorithm-inherited AtomicBlackBox input 
#define BBTK_DECLARE_VTK_INPUT(NAME,TYPE)			\
  public:								\
  TYPE bbGetInput##NAME ()						\
  { return dynamic_cast<TYPE>(vtkParent::GetInput()); }			\
  void bbSetInput##NAME (TYPE d)					\
  { vtkParent::SetInput( (vtkDataObject*) d); /*vtkParent::GetOutput() = d;*/ }

//===========================================================================

//===========================================================================
/// Declares an AtomicBlackBox input corresponding to an 
/// inherited vtk parameter
/// which was declared by vtkSetMacro/vtkGetMacro
/// The NAME **MUST** be the same than the vtk parameter name
#define BBTK_DECLARE_VTK_PARAM(NAME,TYPE)			\
  public:								\
  TYPE bbGetInput##NAME ()						\
  { return vtkParent::Get##NAME(); }					\
  void bbSetInput##NAME (TYPE d)					\
  { vtkParent::Set##NAME(d);						\
    /*bbSetModifiedStatus();*/ }
//===========================================================================



//============================================================================
/// Defines the default bbUserProcess method for vtk inherited black boxes
/// (calls vtkParent::Update)
#define BBTK_VTK_PROCESS()						\
  public:								\
  inline void bbUserProcess()						\
  {									\
    bbtkDebugMessage("process",1,"**> Processing ["<<bbGetFullName()	\
		     <<"]"<<std::endl);					\
    vtkParent::Update();						\
    bbtkDebugMessage("process",2,"<** Processing ["<<bbGetFullName()	\
		     <<"]"<<std::endl);					\
  }
//============================================================================


//===========================================================================
/// EOF
//===========================================================================
#endif
