/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkWxBlackBox.h,v $
  Language:  C++
  Date:      $Date: 2008/12/11 15:30:04 $
  Version:   $Revision: 1.24 $
========================================================================*/


/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         
/**
 * \brief Short description in one line
 * 
 * Long description which 
 * can span multiple lines
 */
/**
 * \file 
 * \brief 
 */
/**
 * \class bbtk::
 * \brief 
 */


#ifdef USE_WXWIDGETS


#ifndef __bbtkWxBlackBox_h__
#define __bbtkWxBlackBox_h__


#include "bbtkWx.h"
#include "bbtkAtomicBlackBox.h"


namespace bbtk
{




  //==================================================================
  // Forward declaration of the class of window associated to a WxBlackBox
  class WxBlackBoxWindow;
  //==================================================================

  //==================================================================
  // Forward declaration of the widget event handler class
  class WxBlackBoxWidgetEventHandler;
  //==================================================================


  //==================================================================
  /// Widget black boxes
  class BBTK_EXPORT WxBlackBox : public bbtk::AtomicBlackBox  
  { 
    BBTK_BLACK_BOX_INTERFACE(WxBlackBox,bbtk::AtomicBlackBox);
    BBTK_DECLARE_INPUT(WinTitle,std::string);
    BBTK_DECLARE_INPUT(WinWidth,int);
    BBTK_DECLARE_INPUT(WinHeight,int);
    BBTK_DECLARE_INPUT(WinDialog,bool);
    BBTK_DECLARE_INPUT(WinHide,Void);
    BBTK_DECLARE_INPUT(WinClose,Void);
    BBTK_DECLARE_OUTPUT(Widget, wxWindow*);

  public:
    /// Main processing method of the box. Overloaded to handle windows inclusion : if the output Widget is connected then the execution is transfered to the box to which it is connected (as the container window must be created and displayed - this box will be executed by the normal pipeline recursion mechanism)
    //   virtual void bbExecute(bool force = false);


    typedef WxBlackBoxWindow Window;
    
    /// Returns the **OWN** window associated to the box
    /// If 0 returned = no window
    Window* bbGetWindow() { return bbmWindow; }

    /// Returns the window containing the widget associated to the box
    /// Which can be the own window of **ANOTHER** box in case of 
    /// a hierarchy of widgets.
    /// More precisely :
    /// If bbGetWindow() != 0 then returns bbGetWindow()
    /// Else if the output 'Widget' is connected 
    ///  then returns bbGetContainingWindow() of the box connected to 'Widget'
    /// Else returns 0;
    Window* bbGetContainingWindow();

    /// Returns true iff the 'containing window' exists and is shown 
    /// (see bbGetContainingWindow).
    bool bbIsShown();

    //==================================================================    
    /// User callback invoked when the containing window is shown
    virtual void bbUserOnShow() {}
    //==================================================================    
 
    //==================================================================    
    /// User callback invoked when the containing window is hidden
    virtual void bbUserOnHide() {}
    //==================================================================    


    //==================================================================    
    void bbCreateWidgetAndEventHandler(wxWindow* parent); 
    //==================================================================    
	  
  protected:
    
  
    //==================================================================
    /// User callback called in the box contructor
    virtual void bbUserConstructor();
    /// User callback called in the box copy constructor
    virtual void bbUserCopyConstructor(bbtk::BlackBox::Pointer);
    /// User callback called in the box destructor
    virtual void bbUserDestructor();
    //==================================================================    

    //==================================================================    
    /// User callback for creating the widget associated to the box
    /// ** Must be defined **
    virtual void bbUserCreateWidget(wxWindow* parent) 
    {
      bbtkError(bbGetTypeName()<<" is a WxBlackBox whose bbUserCreateWidget methods is not overloaded : is it a feature or a bug ?!?");
    }
    //==================================================================    


    //==================================================================    
    // For layout widgets : creates and returns the widget 
    // of the box connected to input in
    wxWindow* bbCreateWidgetOfInput(const std::string& in, wxWindow* parent);
    //==================================================================    

 

    //==================================================================
    /// Overloaded processing method for WxBlackBoxes which handles 
    /// the window creation if needed
    virtual void bbProcess();
    //==================================================================

    //==================================================================
    /// If necessary creates the WxBlackBoxWindow associated to the box
    /// and shows it 
    /// (does nothing if the box output 'Widget' is connected which 
    /// means that the box does not have its own window but is contained 
    /// into another window)
    //    void bbShowWindow();
    /// Hides the WxBlackBoxWindow associated to the box (if exists)
    void bbHideWindow();
    /// Closes (destroys) the WxBlackBoxWindow associated to the box (if exists)
    void bbCloseWindow();
    //==================================================================

  private:
    /// friendship
    friend class WxBlackBoxWindow;
    friend class WxBlackBoxWidgetEventHandler;

    /// Sets the window
    inline void bbSetWindow(Window* w) { bbmWindow=w; }

    /// Sets the Widget Event Handler
    inline void bbSetWidgetEventHandler(WxBlackBoxWidgetEventHandler* w) 
    { bbmWidgetEventHandler = w; }
    /// Gets the Widget Event Handler
    inline WxBlackBoxWidgetEventHandler* bbGetWidgetEventHandler()
    { return bbmWidgetEventHandler; }


    /// The WxBlackBoxWindow associated to the box
    Window* bbmWindow;
    /// The WxBlackBoxWidgetEventHandler associated to the box
    WxBlackBoxWidgetEventHandler* bbmWidgetEventHandler;


    void bbInitAttributes();

  protected :


    /// For Forward update mechanism when execution is called 
    /// on a contained window
    /// Is set to true before transfering update to parent 
    /// in order to not re-transfer a second time...
    /*
    bool bbmUpdateTransferedToParent;

    bool bbGetUpdateTransferedToParent() const { return bbmUpdateTransferedToParent; }
    void bbSetUpdateTransferedToParent(bool b) 
    { bbmUpdateTransferedToParent = b; }
    */
  };
  //=================================================================
 

  //======================================================================
  /// Defines the bbUserCreateWidget method
#define BBTK_CREATE_WIDGET(CALLBACK)					\
  public:								\
  inline void bbUserCreateWidget(wxWindow* parent)					\
  {									\
    bbtkDebugMessageInc("wx",1,"**> Creating widget for ["		\
			<<bbGetFullName()<<"]"<<std::endl);		\
    CALLBACK(parent);								\
    bbtkDebugMessageInc("wx",1,"<** Creating widget for ["		\
			<<bbGetFullName()<<"]"<<std::endl);		\
  }
  
  //======================================================================

  //======================================================================
  /// Defines the bbUserOnShow method
#define BBTK_ON_SHOW_WIDGET(CALLBACK)					\
  public:								\
  inline void bbUserOnShow()					\
  {									\
    bbtkDebugMessageInc("wx",1,"**> Showing ["		\
			<<bbGetFullName()<<"]"<<std::endl);		\
    CALLBACK();								\
    bbtkDebugMessageInc("wx",1,"<** Showing ["		\
			<<bbGetFullName()<<"]"<<std::endl);		\
  }
  
  //======================================================================

  //=================================================================
  // WxBlackBoxDescriptor declaration
  BBTK_BEGIN_DESCRIBE_BLACK_BOX(WxBlackBox,bbtk::AtomicBlackBox);
  BBTK_NAME("WxBlackBox");
  // BBTK_DESCRIPTION("Widget box. The inputs marked with (*) are only used if the widget is not inserted in another widget.\n");
  BBTK_CATEGORY("widget");
  BBTK_INPUT(WxBlackBox,WinTitle,
	     "Title of the window (*)",
	     std::string,"");
  BBTK_INPUT(WxBlackBox,WinWidth,
	     "Width of the window (* : only used if the widget is not connected to a Layout box)",int,"");
  BBTK_INPUT(WxBlackBox,WinHeight,
	     "Height of the window (*)",int,"");
  BBTK_INPUT(WxBlackBox,WinDialog,
	     "Set to 'true' to create a dialog window, i.e. which blocks the pipeline until it is closed (modal) (*)",bool,"");
  BBTK_INPUT(WxBlackBox,WinHide,
	     "Any signal received hides the window (*)",Void,"signal");
  BBTK_INPUT(WxBlackBox,WinClose,
	     "Any signal received closes the window (*)",Void,"signal");
  BBTK_OUTPUT(WxBlackBox,Widget,"Output widget",wxWindow*,"");
  BBTK_END_DESCRIBE_BLACK_BOX(WxBlackBox);
  //=================================================================









  //==================================================================
  // The base of the hierarchy of windows associated to a WxBlackBox
  class BBTK_EXPORT WxBlackBoxWindow //: public wxWindow
  {
  public:
    WxBlackBoxWindow(WxBlackBox::Pointer box);
    virtual ~WxBlackBoxWindow();
    virtual void bbShow();
    virtual void bbHide();
    virtual void bbClose();
    bool bbIsShown() { return mShown; }
    virtual WxBlackBox::Pointer bbGetBlackBox() { return mBox.lock(); }
    virtual wxDialog* bbGetDialog() { return 0; } 
    virtual wxFrame* bbGetFrame() { return 0; } 
  private:
    WxBlackBox::WeakPointer mBox;
    bool mShown;
  };
  //==================================================================

  //==================================================================
  // Dialog window which is modal
  class BBTK_EXPORT  WxBlackBoxDialog : public wxDialog, public WxBlackBoxWindow
  {
  public:
    WxBlackBoxDialog(WxBlackBox::Pointer box, 
		     wxWindow *parent, wxString title, wxSize size);
    ~WxBlackBoxDialog();
    void bbShow();  
    void bbHide();
    void bbClose();
    wxDialog* bbGetDialog() { return this; } 
  };
  //==================================================================

  //==================================================================
  // Frame window which is not modal
  class BBTK_EXPORT  WxBlackBoxFrame : public wxFrame, public WxBlackBoxWindow
  {
  public:
    WxBlackBoxFrame(WxBlackBox::Pointer box,
		    wxWindow *parent, wxString title, wxSize size);
    ~WxBlackBoxFrame();
    void bbShow();
    void bbHide();
    void bbClose();
    wxFrame* bbGetFrame() { return this; } 
  };
  //==================================================================



  //=================================================================
  // Handles the destroy events of a widget associated to a WxBlackBox 
  // in order to signal the widget death to its associated box
  class BBTK_EXPORT WxBlackBoxWidgetEventHandler : public wxEvtHandler
  {
  public:
    /// Ctor with the box and widget 
    WxBlackBoxWidgetEventHandler( WxBlackBox::Pointer box, wxWindow *widget );
    /// Dtor
    ~WxBlackBoxWidgetEventHandler();
    /// Returns true iff is the handler for that window  
    bool IsHandlerOf( wxWindow* w ) { return mWindow == w; }
    // wxWindow* GetWxWindow() { return mWindow; }
    /// Method processing the destroy event of the widget
    void OnWindowDestroy(wxWindowDestroyEvent&);
    //
    //bool IsDead() { return mDead; }

  private:
    WxBlackBox::WeakPointer mBox;
    wxWindow* mWindow;
    //bool mDead;
  };  
  //=================================================================



} //namespace bbtk

#endif  //__bbtkWxBlackBox_h__

#endif  //USE_WXWIDGETS
