/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbitkExtractImageFilter.h,v $
  Language:  C++
  Date:      $Date: 2008/11/12 12:47:03 $
  Version:   $Revision: 1.9 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


#ifdef _USE_ITK_

#include "bbtkAtomicBlackBox.h"
#include "itkExtractImageFilter.h"
#include "bbtkItkBlackBoxMacros.h"
#include "bbitkImage.h"
#include "bbitkImageRegion.h"

namespace bbitk
{

  
  template <class T>
  class /*BBTK_EXPORT*/ ExtractImageFilter
    : 
    public bbtk::AtomicBlackBox,
    public itk::ExtractImageFilter<T,T>
  {
    BBTK_TEMPLATE_BLACK_BOX_INTERFACE(ExtractImageFilter,
				      bbtk::AtomicBlackBox,
				      T);
    typedef itk::ExtractImageFilter<T,T> itkParent;

    BBTK_DECLARE_ITK_INPUT(In,const T*);
    BBTK_DECLARE_ITK_PARAM(ExtractionRegion,typename T::RegionType);
    BBTK_DECLARE_ITK_OUTPUT(Out,T*);

    BBTK_ITK_PROCESS();
    BBTK_ITK_DELETE();
  };


  BBTK_BEGIN_DESCRIBE_TEMPLATE_BLACK_BOX(ExtractImageFilter,
				      bbtk::AtomicBlackBox);
  BBTK_NAME("ExtractImageFilter<"+bbtk::TypeName<T>()+">");
  BBTK_AUTHOR("laurent.guigues@creatis.insa-lyon.fr");
  BBTK_DESCRIPTION("Decrease the image size by cropping the image to the selected region bounds (bbification of itk::ExtractImageFilter)");
  BBTK_TEMPLATE_INPUT(ExtractImageFilter,In,"Input image",const T*);
  BBTK_TEMPLATE_INPUT(ExtractImageFilter,ExtractionRegion,"Extraction region",typename T::RegionType);
  BBTK_TEMPLATE_OUTPUT(ExtractImageFilter,Out,"Output image",T*);
  BBTK_END_DESCRIBE_TEMPLATE_BLACK_BOX(ExtractImageFilter);
  
  //===================================================
  // Generic "untemplatized" filter
  //===================================================
  class /*BBTK_EXPORT*/ ExtractImageFilterGeneric
    : public bbtk::AtomicBlackBox
  {
    BBTK_BLACK_BOX_INTERFACE(ExtractImageFilterGeneric,
			     bbtk::AtomicBlackBox);
    BBTK_DECLARE_INPUT(In,anyImagePointer);
    BBTK_DECLARE_INPUT(Region,anyImageRegion);
    BBTK_DECLARE_OUTPUT(Out,anyImagePointer);
    BBTK_PROCESS(ProcessSwitch);
  private :
    inline void ProcessSwitch();
    template <class T> void Process();
  };
  
  BBTK_BEGIN_DESCRIBE_BLACK_BOX(ExtractImageFilterGeneric,
				bbtk::AtomicBlackBox);
  BBTK_NAME("ExtractImageFilter");
  BBTK_AUTHOR("laurent.guigues at creatis.insa-lyon.fr");
  BBTK_CATEGORY("image;filter");
  BBTK_DESCRIPTION("Decrease the image size by cropping the image to the selected region bounds (bbification of itk::ExtractImageFilter)");
  BBTK_INPUT(ExtractImageFilterGeneric,In,"Input image",anyImagePointer,"");
  BBTK_INPUT(ExtractImageFilterGeneric,Region,"Extraction region",anyImageRegion,"");
  BBTK_OUTPUT(ExtractImageFilterGeneric,Out,"Output image",anyImagePointer,"");
  BBTK_END_DESCRIBE_BLACK_BOX(ExtractImageFilterGeneric);
  void ExtractImageFilterGeneric::ProcessSwitch()
  {
     BBTK_TEMPLATE_ITK_IMAGE_SWITCH(bbGetInputIn().type(), this->Process);
  }

  template <class T>
  void ExtractImageFilterGeneric::Process()
  {
    bbtkDebugMessageInc("Core",9,
			"bbitk::ExtractImageFilterGeneric::Process<"
			<<bbtk::TypeName<T>()<<">()"<<std::endl);
    typedef T ImageType;
    typedef ExtractImageFilter<ImageType> FilterType;
    typename FilterType::Pointer f = FilterType::New("Temp");
	  typedef ImageType* ImageTypePointer; 
    f->bbSetInputIn( this->bbGetInputIn().unsafe_get<ImageTypePointer>() );
    f->bbSetInputExtractionRegion ( this->bbGetInputRegion().get<typename T::RegionType>() );
    f->bbExecute();
    f->bbGetOutputOut()->Register();
    this->bbSetOutputOut( f->bbGetOutputOut() );
    //  f->bbDelete();

    bbtkDebugDecTab("Core",9);
  }

}
// EO namespace bbitk

#endif 
