/*=========================================================================
                                                                                
  Program:   bbtk
  Module:    $RCSfile: bbitkvtkvtkImageData2itkImage.cxx,v $
  Language:  C++
  Date:      $Date: 2008/11/25 11:17:18 $
  Version:   $Revision: 1.4 $
                                                                                
  Copyright (c) CREATIS (Centre de Recherche et d'Applications en Traitement de
  l'Image). All rights reserved. See Doc/License.txt or
  http://www.creatis.insa-lyon.fr/Public/bbtk/License.html for details.
                                                                                
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.
                                                                                
=========================================================================*/
/**
 *  \file 
 *  \brief 
 */
#ifdef _USE_ITK_
#ifdef _USE_VTK_

#include "bbitkvtkvtkImageData2itkImage.h"
#include "bbitkvtkPackage.h"
// VtkToItkConnection 
#include "itkVTKImageToImageFilter.h"
// ItkToVtkConnection
//#include "itkImageToVTKImageFilter.h"


namespace bbitkvtk 
{
  
  BBTK_BLACK_BOX_IMPLEMENTATION(vtkImageData2itkImage,bbtk::AtomicBlackBox);

  BBTK_ADD_BLACK_BOX_TO_PACKAGE(itkvtk,vtkImageData2itkImage);
  
#define BBTK_TEMPLATE_VTK_IMAGE_DATA_SWITCH_DIM(I,DIM,M)		\
  if (I->GetDataDimension()==DIM)					\
    {									\
      if (I->GetScalarType()==VTK_CHAR) M<char,DIM>();			\
      else if (I->GetScalarType()==VTK_SIGNED_CHAR) M<signed char,DIM>(); \
      else if (I->GetScalarType()==VTK_UNSIGNED_CHAR) M<unsigned char,DIM>(); \
      else if (I->GetScalarType()==VTK_SHORT) M<short,DIM>();		\
      else if (I->GetScalarType()==VTK_UNSIGNED_SHORT) M<unsigned short,DIM>(); \
      else if (I->GetScalarType()==VTK_INT) M<int,DIM>();		\
      else if (I->GetScalarType()==VTK_UNSIGNED_INT) M<unsigned int,DIM>(); \
      else if (I->GetScalarType()==VTK_LONG) M<long,DIM>();		\
      else if (I->GetScalarType()==VTK_UNSIGNED_LONG) M<unsigned long,DIM>(); \
      else if (I->GetScalarType()==VTK_FLOAT) M<float,DIM>();		\
      else if (I->GetScalarType()==VTK_DOUBLE) M<double,DIM>();		\
    }
  
#define BBTK_TEMPLATE_VTK_IMAGE_DATA_SWITCH(I,M) \
  BBTK_TEMPLATE_VTK_IMAGE_DATA_SWITCH_DIM(I,2,M) \
  BBTK_TEMPLATE_VTK_IMAGE_DATA_SWITCH_DIM(I,3,M) 

  void vtkImageData2itkImage::Convert()
  {   
    BBTK_TEMPLATE_VTK_IMAGE_DATA_SWITCH(bbGetInputIn(),Convert)
  }
  


   template<class T, unsigned int D>
   void vtkImageData2itkImage::Convert()
  {
    bbtkDebugMessage("process",5,"==> ["<<bbGetFullName()<<"] : Convert<"
			<<bbtk::TypeName<T>()<<","<<D
			<<">()"<<std::endl);
    

    typedef itk::Image<T,D> itkImageType;
    typedef itk::VTKImageToImageFilter< itkImageType > VtkToItkConnection;
    typename VtkToItkConnection::Pointer conv;

    // No converter yet : create it and set its input
    if (!mConverter) 
      {
	bbtkDebugMessage("process",5,"    ["<<bbGetFullName()<<"] : No converter yet : creating it"<<std::endl);
	conv = VtkToItkConnection::New();
	mConverter = conv;
	conv->SetInput( this->bbGetInputIn() );
      }
    else 
      {
	bbtkDebugMessage("process",5,"    ["<<bbGetFullName()<<"] : Converter ok "<<std::endl);
	// Input vtkImage type changed ? 
	// Have to change the converter and set its input
	conv = dynamic_cast<VtkToItkConnection*>
	  ((itk::ProcessObject*)mConverter);
	if (!conv) 
	  {
	    bbtkDebugMessage("process",5,"    ["<<bbGetFullName()<<"] : Converter of wrong input type : reacreating it "<<std::endl);
	    mConverter->UnRegister();
	    conv = VtkToItkConnection::New();
	    mConverter = conv;
	    conv->SetInput( this->bbGetInputIn() );
	  }
	// Input image type did not change but input image pointer did:
	// set new input
	else if ( this->bbGetInputIn() != conv->GetExporter()->GetInput())
	  {
	    bbtkDebugMessage("process",5,"    ["<<bbGetFullName()<<"] : Converter input changed : resetting it"<<std::endl);
	    conv->SetInput( this->bbGetInputIn() );
	  }
	else 
	  {
	    bbtkDebugMessage("process",5,"    ["<<bbGetFullName()<<"] : Nothing changed"<<std::endl);	  
	  }
      }
    try
      {
	
	bbtkDebugMessage("process",5,"    ["<<bbGetFullName()<<"] : Trying update"<<std::endl);	 
	conv->Update();
	itkImageType* out = const_cast<itkImageType*>(conv->GetOutput());
	bbSetOutputOut(out);	
      }
    catch( itk::ExceptionObject & e) 
      {
	bbtkError("vtkImageData2itkImage<"
		  <<bbtk::TypeName<T>()<<","<<D
		  <<">::Convert() : "<<e);
      } 
    bbtkDebugMessage("process",5,"<== ["<<bbGetFullName()<<"] : Convert<"
		     <<bbtk::TypeName<T>()<<","<<D
		     <<">() *DONE*"<<std::endl);	 
  }

  
  void vtkImageData2itkImage::bbUserConstructor()
  {
    //    bbSetOutputOut(NULL);
    mConverter = 0;
  }
  
  void vtkImageData2itkImage::bbUserCopyConstructor(bbtk::BlackBox::Pointer)
  {
    //    bbSetOutputOut(NULL);
    mConverter = 0;
  }
  
  void vtkImageData2itkImage::bbUserDestructor()
  {
    //    if (mConverter) mConverter->UnRegister();
  }
  



}
// eo namespace bbtk

#endif
// _USE_VTK_
#endif
// _USE_ITK_
