/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbvtkIsoSurfaceExtractor.h,v $
  Language:  C++
  Date:      $Date: 2008/11/25 11:17:23 $
  Version:   $Revision: 1.11 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 * \brief Short description in one line
 * 
 * Long description which 
 * can span multiple lines
 */
/**
 * \file 
 * \brief Pattern for the definition of a new type of Node (header)
 */
/**
 * \class bbtk::NodePatern 
 * \brief Pattern for the definition of a new type of Node 
 */


#ifdef _USE_VTK_


#ifndef __vtkIsoSurfaceExtractor_h__
#define __vtkIsoSurfaceExtractor_h__

#include "vtkImageData.h"
#include "vtkRenderer.h"
#include "vtkMarchingCubes.h"
#include "vtkPolyDataMapper.h"
#include "vtkProp3D.h"
#include "vtkActor.h"
#include <vector>

#include "bbtkAtomicBlackBox.h"

namespace bbvtk
{
  
  class /*BBTK_EXPORT*/ IsoSurfaceExtractor
    : 
    public bbtk::AtomicBlackBox
  {
    bool              firsttime;
    vtkMarchingCubes  *marchingcubes;
    vtkPolyDataMapper *polydatamapper;
    vtkActor          *vtkactor; 
    
    
    BBTK_BLACK_BOX_INTERFACE(IsoSurfaceExtractor,bbtk::AtomicBlackBox);
    
    BBTK_DECLARE_INPUT(In,vtkImageData *);
    //    BBTK_DECLARE_INPUT(InVtkObject,vtkObject *);
    BBTK_DECLARE_INPUT(Isovalue,double);
    BBTK_DECLARE_INPUT(Opacity,double);
    BBTK_DECLARE_INPUT(Colour,std::vector<double>);
    BBTK_DECLARE_INPUT(Renderer,vtkRenderer *);
	BBTK_DECLARE_INPUT(Transform, vtkLinearTransform *);
    BBTK_DECLARE_OUTPUT(Out,vtkProp3D *);
    
    BBTK_PROCESS(DoProcess);
    void DoProcess();
    
  protected:
    virtual void bbUserConstructor();
    virtual void bbUserCopyConstructor(bbtk::BlackBox::Pointer);
    virtual void bbUserDestructor();
    void Init();
  };
  
  BBTK_BEGIN_DESCRIBE_BLACK_BOX(IsoSurfaceExtractor,bbtk::AtomicBlackBox);
  BBTK_NAME("IsoSurfaceExtractor");
  BBTK_AUTHOR("eduardo.davila / laurent.guigues at creatis.insa-lyon.fr");
  BBTK_DESCRIPTION("Extracts an iso-surface of a 3D image and creates a vtkProp3D object to insert into a 3D scene (e.g. a Viewer3D)");
  typedef std::vector<double> vectorcolour;
  BBTK_CATEGORY("3D object creator");
  BBTK_INPUT(IsoSurfaceExtractor,In,"Input image",vtkImageData*,"");
  //  BBTK_INPUT(IsoSurfaceExtractor,InVtkObject,"Image vtkObject",vtkObject*);

  BBTK_INPUT(IsoSurfaceExtractor,Isovalue,"Isovalue",double,"");
  BBTK_INPUT(IsoSurfaceExtractor,Opacity,"Opacity",double,"");
  BBTK_INPUT(IsoSurfaceExtractor,Colour,"r g b",vectorcolour,"colour");
  BBTK_INPUT(IsoSurfaceExtractor,Renderer,"3D scene in which to insert the surface",vtkRenderer*,"");
  BBTK_INPUT(IsoSurfaceExtractor,Transform,"Linear Transform (4x4 homogeneous)",vtkLinearTransform*,"");
  BBTK_OUTPUT(IsoSurfaceExtractor,Out,"Extracted iso-surface",vtkProp3D *,"");
  
  BBTK_END_DESCRIBE_BLACK_BOX(IsoSurfaceExtractor);
  
}// EO namespace bbtk


#endif  //__IsoSurfaceExtractor_h__

#endif //_USE_VTK_

